SystemOrganization addCategory: #'XMLRPC-Server'!
SystemOrganization addCategory: #'XMLRPC-Server-Errors'!
SystemOrganization addCategory: #'XMLRPC-Server-Test'!
SystemOrganization addCategory: #'XMLRPC-Server-Validation'!

!SmallInteger methodsFor: '*xmlrpc-server' stamp: 'maf 2/27/2005 23:24'!
asXMLRPCString
	"answer a XMLRPC representation of the receiver"
	| w |
	w _ WriteStream on: (String new: 50).
	w nextPutAll: '<value><i4>'; nextPutAll: self asString; nextPutAll: '</i4></value>'.
	^ w contents! !

TestCase subclass: #XMLRPCServerTest
	instanceVariableNames: 'newServer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XMLRPC-Server-Test'!

!XMLRPCServerTest methodsFor: 'as yet unclassified' stamp: 'maf 3/1/2005 19:17'!
setUp
	newServer := false.
	XMLRPCStandardValidator addTests.
	(HttpService servicesNamed: 'xmlrpc') 
		ifEmpty: [XMLRPCHttpModule start; setDebugMode.
			newServer := true]! !

!XMLRPCServerTest methodsFor: 'as yet unclassified' stamp: 'maf 3/1/2005 13:51'!
tearDown
	#('validator1.arrayOfStructsTest' 'validator1.countTheEntities' 'validator1.echoStructTest' 'validator1.easyStructTest' 'validator1.moderateSizeArrayCheck' 'validator1.nestedStructTest' 'validator1.simpleStructReturnTest' 'validator1.manyTypesTest') do: [ :xmlrpcService |
	XMLRPCServerRequest removeService: xmlrpcService asSymbol].
	newServer ifTrue: [ XMLRPCHttpModule stop ]! !

!XMLRPCServerTest methodsFor: 'as yet unclassified' stamp: 'maf 3/1/2005 19:03'!
testDictionary
	| d p r |
	d := Dictionary new.
	d at: 'true' put: true.
	d at: 'Markus' put: 'Fritsche'.
	p := XMLRPCProxy new url: 'http://localhost:8081/xmlrpc' asUrl.
	r := p invokeMethod: 'validator1.echoStructTest' withArgs: {d}.
	self assert: (r = d)
	
! !

!XMLRPCServerTest methodsFor: 'as yet unclassified' stamp: 'maf 3/1/2005 19:03'!
testHallo
	| p |
	p := XMLRPCProxy new url: 'http://localhost:8081/xmlrpc' asUrl.
	p invokeMethod: 'validator1.echoStructTest' withArgs: #('Hallo')
! !

!Float methodsFor: '*xmlrpc-server' stamp: 'maf 2/27/2005 23:21'!
asXMLRPCString
	"answer a XMLRPC representation of the receiver"
	| w | 
	w := WriteStream
				on: (String new: 50).
	w nextPutAll: '<value><double>';
		 nextPutAll: self asString;
		 nextPutAll: '</double></value>'.
	^ w contents! !

!True methodsFor: '*xmlrpc-server' stamp: ''!
asXMLRPCString
	"answer a XMLRPC representation of the receiver"
	^ '<value><boolean>1</boolean></value>
'! !

!Dictionary methodsFor: '*xmlrpc-server' stamp: 'maf 2/27/2005 23:20'!
asXMLRPCString
	"answer a XMLRPC representation of the receiver"
	| w |
	w := WriteStream
				on: (String new: 200).
	w nextPutAll: '<value><struct>'.
	self
		keysAndValuesDo: [:key :value | 
			w nextPutAll: '<member><name>';
				 nextPutAll: key asString;
				 nextPutAll: '</name>'.
			w nextPutAll: value asXMLRPCString;
				 nextPutAll: '</member>'].
	w nextPutAll: '</struct></value>'.
	^ w contents! !

!False methodsFor: '*xmlrpc-server' stamp: 'maf 2/27/2005 23:20'!
asXMLRPCString
	"answer a XMLRPC representation of the receiver"
	^ '<value><boolean>0</boolean></value>'! !

!XMLRPCDateTime methodsFor: '*xmlrpc-server' stamp: 'mfrit 11/16/2002 20:05'!
asXMLRPCString
	| str |
	str _ WriteStream on: (String new: 70).
	str nextPutAll: '<value><dateTime.iso8601>'; nextPutAll:
	self encodeISO8601; nextPutAll: '</dateTime.iso8601></value>';cr.
	^ str contents! !

!SequenceableCollection methodsFor: '*xmlrpc-server' stamp: 'maf 2/27/2005 23:23'!
asXMLRPCString
	"answer a XMLRPC representation of the receiver"
	| w |
	w := WriteStream on: (String new: 200).
	w nextPutAll: '<value><array><data>'.
	self
		do: [:each | w nextPutAll: each asXMLRPCString].
	w nextPutAll: '</data></array></value>'.
	^ w contents! !

!Stream methodsFor: '*xmlrpc-server' stamp: 'maf 2/27/2005 23:25'!
asXMLRPCString
	"answer a XMLRPC representation of the receiver. A stream will be
	encoded as an base64 (mimeencoded) element."
	| w |
	w := WriteStream on: String new.
	w nextPutAll: '<value><base64>';
		 nextPutAll: (Base64MimeConverter mimeEncode: self) contents;
		 nextPutAll: '</base64></value>'.
	^ w contents! !

!Object methodsFor: '*xmlrpc-server' stamp: ''!
asXMLRPCString
	"answer a XMLRPC struct that includes all instanceVariables of the 
	receiver. A XMLRPC struct is represented as a dictionary in squeak"
	| d |
	d _ Dictionary new.
	self class instVarNames
		do: [:each | d
				at: each asSymbol
				put: (self instVarNamed: each)].
	d size > 0 ifTrue: [^ d asXMLRPCString].
	^'<value>nil</value>'! !

Object subclass: #XMLRPCErrorHandler
	instanceVariableNames: 'error'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XMLRPC-Server'!

!XMLRPCErrorHandler commentStamp: '<historical>' prior: 0!
This is the error handler for XMLRPC errors!

!XMLRPCErrorHandler methodsFor: 'accessing' stamp: ''!
error
	"Answer the receiver's 'error'."

	^error! !

!XMLRPCErrorHandler methodsFor: 'accessing' stamp: ''!
error: anObject
	"Set the receiver's instance variable 'error' to be anObject."

	error := anObject! !

!XMLRPCErrorHandler methodsFor: 'accessing' stamp: 'mfrit 11/16/2002 18:58'!
errorMessage
	| str errorDict |
	errorDict _ Dictionary new.
	(error class canUnderstand: #faultCode)
		ifTrue: [errorDict at: #faultCode put: error faultCode]
		ifFalse: [errorDict at: #faultCode put: 10]. 
	errorDict at: #faultString put: error messageText.
	str _ WriteStream
				on: (String new: 1000).
	str nextPutAll: '<?xml version="1.0"?><methodResponse><fault>';
		 nextPutAll: errorDict asXMLRPCString;
		 nextPutAll: '</fault></methodResponse>'.
	^ str contents! !

Object subclass: #XMLRPCHttpModule
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XMLRPC-Server'!

XMLRPCHttpModule class
	instanceVariableNames: 'port service'!

!XMLRPCHttpModule commentStamp: '<historical>' prior: 0!
This comanche module handles incoming requests for a XMLRPC server!

XMLRPCHttpModule class
	instanceVariableNames: 'port service'!

!XMLRPCHttpModule classSide methodsFor: 'as yet unclassified' stamp: 'maf 3/1/2005 18:56'!
createServiceOnPort: aPort
	| serv xs ma |
	Smalltalk
		at: #HttpService
		ifPresent: [:hs | 
			xs := XMLRPCHttpModule new.
			ma := ModuleAssembly core.
			ma
				alias: '/xmlrpc'
				to: [ma
						addPlug: [:request | xs process: request]].
			serv := HttpService on: aPort named: 'xmlrpcd'.
			serv plug: ma rootModule.
			^ serv].
	Smalltalk
		at: #ComancheNetService
		ifPresent: [:cns | ^ (cns named: 'xmlrpc' onPort: port)
				module: self new]! !

!XMLRPCHttpModule classSide methodsFor: 'as yet unclassified' stamp: ''!
initialize
	Smalltalk addToStartUpList: self after: AutoStart.
	Smalltalk addToShutDownList: self! !

!XMLRPCHttpModule classSide methodsFor: 'as yet unclassified' stamp: 'maf 3/1/2005 13:29'!
pause
	service ifNotNil:
		[service unregister.
		service := nil]! !

!XMLRPCHttpModule classSide methodsFor: 'as yet unclassified' stamp: 'maf 3/1/2005 13:29'!
setDebugMode
	service setDebugMode.! !

!XMLRPCHttpModule classSide methodsFor: 'as yet unclassified' stamp: 'maf 3/1/2005 13:29'!
setDeploymentMode
	service setDeploymentMode.! !

!XMLRPCHttpModule classSide methodsFor: 'as yet unclassified' stamp: 'maf 3/1/2005 13:29'!
setVerboseMode
	service setVerboseMode.! !

!XMLRPCHttpModule classSide methodsFor: 'as yet unclassified' stamp: 'maf 3/1/2005 13:30'!
shutdown
	self pause! !

!XMLRPCHttpModule classSide methodsFor: 'as yet unclassified' stamp: 'maf 3/1/2005 19:10'!
start
	^ self startOn: 8081! !

!XMLRPCHttpModule classSide methodsFor: 'as yet unclassified' stamp: 'maf 3/1/2005 17:11'!
startKom61
	^ self start! !

!XMLRPCHttpModule classSide methodsFor: 'as yet unclassified' stamp: 'maf 3/1/2005 19:15'!
startOn: aPort
	self stop.
	port := aPort.
	service := self createServiceOnPort: aPort.
	service start! !

!XMLRPCHttpModule classSide methodsFor: 'as yet unclassified' stamp: 'maf 3/1/2005 13:46'!
startUp
	port ifNotNil: [self startOn: port]! !

!XMLRPCHttpModule classSide methodsFor: 'as yet unclassified' stamp: 'maf 3/1/2005 13:29'!
stop
	self pause.
	port := nil.
! !

!XMLRPCHttpModule methodsFor: 'as yet unclassified' stamp: 'mfrit 11/25/2002 15:36'!
process: request 
	| xrReq |
	[xrReq _ XMLRPCServerRequest
				fromRequest: (request stream next: request contentLength).
	xrReq targetURI: request url.
	^ HttpResponse
		fromMIMEDocument: (MIMEDocument contentType: 'text/xml' content: xrReq methodResponse)]
		on: Error
		do: [:e | ^ HttpResponse
				fromMIMEDocument: (MIMEDocument contentType: 'text/xml' content: (XMLRPCErrorHandler new error: e) errorMessage)]! !

Object subclass: #XMLRPCServerRequest
	instanceVariableNames: 'methodCall params targetURI'
	classVariableNames: 'Receivers'
	poolDictionaries: ''
	category: 'XMLRPC-Server'!

!XMLRPCServerRequest commentStamp: '<historical>' prior: 0!
This class holds all information about a XMLRPC request an coordinates the computation of a XMLRPC methodcall!

!XMLRPCServerRequest classSide methodsFor: 'as yet unclassified' stamp: ''!
cleanUpReceiverDictionary
	" deletes references to obsolete classes from the 'Receivers' dictionary"
	| removeThem |
	removeThem _ Receivers
				select: [:each | each isObsolete].
	removeThem
		keysDo: [:key | Receivers removeKey: key]! !

!XMLRPCServerRequest classSide methodsFor: 'as yet unclassified' stamp: 'mfrit 11/16/2002 19:22'!
fromRequest: xmlString 
	| xmldoc r parameters |
	r _ self new.
	parameters _ OrderedCollection new.
	xmldoc _ XMLDOMParser parseDocumentFrom: xmlString readStream.
	r methodCall: (xmldoc topElement elementAt: #methodName) contentString.
	[((xmldoc topElement)
		elementAt: #params) elements
		do: [:each | parameters
				add: (XMLRPCDecoder new decode: each)]] 
		on: Error
		do: [XMLRPCRequestError new signal: 'Could not decode request'].
	r params: parameters.
	^ r! !

!XMLRPCServerRequest classSide methodsFor: 'as yet unclassified' stamp: ''!
initialize
	Receivers _ Dictionary new.! !

!XMLRPCServerRequest classSide methodsFor: 'as yet unclassified' stamp: 'mfrit 11/25/2002 15:25'!
registerService: serviceName class: aClass selector: aSelector 
	Receivers
		ifNil: [Receivers _ Dictionary new].
	Receivers at: serviceName asSymbol put: aClass -> aSelector! !

!XMLRPCServerRequest classSide methodsFor: 'as yet unclassified' stamp: 'mfrit 11/26/2002 00:27'!
removeService: aXMLRPCService
	Receivers removeKey: aXMLRPCService asSymbol! !

!XMLRPCServerRequest methodsFor: 'accessing' stamp: ''!
methodCall
	"Answer the receiver's 'methodCall'."

	^methodCall! !

!XMLRPCServerRequest methodsFor: 'accessing' stamp: ''!
methodCall: anObject
	"Set the receiver's instance variable 'methodCall' to be anObject."

	methodCall := anObject! !

!XMLRPCServerRequest methodsFor: 'invocation' stamp: 'mfrit 11/25/2002 15:36'!
methodResponse
	| srv stream ret |
	stream _ WriteStream on: String new.
	stream nextPutAll: '<?xml version="1.0"?><methodResponse><params><param>'.
	[
	srv _ (Receivers
				at: methodCall asSymbol
				ifAbsent: [XMLRPCMethodError new signal: 'Serverside method not found']).
	ret _ ((srv key) perform: (srv value) with: (self params)).
]
		on: MessageNotUnderstood
		do: [:e | XMLRPCMethodError new signal: 'Message not understood by  
	server class'].
	ret
		ifNil: [ret _ false].
	[stream nextPutAll: ret asXMLRPCString]
		on: Error
		do: [XMLRPCReturnError new signal: 'Could not encode returned value "' , ret asString , '"'].
	stream nextPutAll: '</param></params></methodResponse>'.
	^ stream contents! !

!XMLRPCServerRequest methodsFor: 'accessing' stamp: ''!
params
	"Answer the receiver's 'params'."

	^params! !

!XMLRPCServerRequest methodsFor: 'accessing' stamp: ''!
params: anObject
	"Set the receiver's instance variable 'params' to be anObject."

	params := anObject! !

!XMLRPCServerRequest methodsFor: 'accessing' stamp: ''!
targetURI
	"Answer the receiver's 'targetURI'."

	^targetURI! !

!XMLRPCServerRequest methodsFor: 'accessing' stamp: ''!
targetURI: anObject
	"Set the receiver's instance variable 'targetURI' to be anObject."

	targetURI := anObject! !

Object subclass: #XMLRPCStandardValidator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XMLRPC-Server-Validation'!

!XMLRPCStandardValidator classSide methodsFor: 'as yet unclassified' stamp: 'mfrit 11/25/2002 15:35'!
addTests
	"XMLRPCStandardValidator addTests"

	XMLRPCServerRequest registerService: 'validator1.arrayOfStructsTest' class: self selector: #arrayOfStructsTest:.
	XMLRPCServerRequest registerService: 'validator1.countTheEntities' class: self selector: #countTheEntities:.
	XMLRPCServerRequest registerService: 'validator1.echoStructTest' class: self selector: #echoStructTest:.
	XMLRPCServerRequest registerService: 'validator1.easyStructTest' class: self  selector: #easyStructTest:.
	XMLRPCServerRequest registerService: 'validator1.moderateSizeArrayCheck' class: self selector: #moderateSizeArrayCheck:.
	XMLRPCServerRequest registerService: 'validator1.nestedStructTest' class: self selector: #nestedStructTest:.
	XMLRPCServerRequest registerService: 'validator1.simpleStructReturnTest' class: self selector: #simpleStructReturnTest:.
	XMLRPCServerRequest registerService: 'validator1.manyTypesTest' class: self selector: #manyTypesTest:! !

!XMLRPCStandardValidator classSide methodsFor: 'as yet unclassified' stamp: ''!
arrayOfStructsTest: aCollection 
	| ret |
	ret _ 0.
	aCollection first	do: [:each | ret _ ret
								+ (each at: 'curly')].
	^ ret! !

!XMLRPCStandardValidator classSide methodsFor: 'as yet unclassified' stamp: 'M.F. 9/6/2002 16:42'!
countTheEntities: aCollection 
	| ret string |
	ret _ Dictionary new.
	string _ aCollection first.
	ret
		at: 'ctLeftAngleBrackets'
		put: (string occurrencesOf: $<).
	ret
		at: 'ctRightAngleBrackets'
		put: (string occurrencesOf: $>).
	ret
		at: 'ctAmpersands'
		put: (string occurrencesOf: $&).
	ret
		at: 'ctApostrophes'
		put: (string occurrencesOf: $').
	ret
		at: 'ctQuotes'
		put: (string occurrencesOf: $").
	^ ret! !

!XMLRPCStandardValidator classSide methodsFor: 'as yet unclassified' stamp: ''!
easyStructTest: aCollection
	| ret |
	ret _ 0. aCollection first valuesDo: [ :each | ret _ ret + each ].
	^ret! !

!XMLRPCStandardValidator classSide methodsFor: 'as yet unclassified' stamp: 'mfrit 12/22/2002 16:01'!
echoStructTest: aCollection 

	^ aCollection first! !

!XMLRPCStandardValidator classSide methodsFor: 'as yet unclassified' stamp: 'mfrit 12/22/2002 16:17'!
manyTypesTest: aCollection
    "simply return the decoded values in aCollection encoded"
	| arr x | 
	x _ 1.
	arr _ Array new: 6.
	aCollection do: [ :each | arr at: x put: each. x_x+1].
	^arr ! !

!XMLRPCStandardValidator classSide methodsFor: 'as yet unclassified' stamp: ''!
moderateSizeArrayCheck: aCollection 
	| arr |
	arr _ aCollection first.
	^ (arr first), (arr last)! !

!XMLRPCStandardValidator classSide methodsFor: 'as yet unclassified' stamp: ''!
nestedStructTest: aCollection
	| struct ret |
	struct _ aCollection first.
	ret _ 0.
	(((struct at: '2000') at: '04') at: '01') valuesDo: [ :each | ret _ ret + each].
	^ ret! !

!XMLRPCStandardValidator classSide methodsFor: 'as yet unclassified' stamp: ''!
simpleStructReturnTest: aCollection 
	| num dict |
	dict _ Dictionary new.
	num _ aCollection first.
	dict at: 'times10' put: num * 10;
		 at: 'times100' put: num * 100;
		 at: 'times1000' put: num * 1000.
	^ dict! !

Error subclass: #XMLRPCError
	instanceVariableNames: 'faultCode'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XMLRPC-Server-Errors'!

!XMLRPCError commentStamp: '<historical>' prior: 0!
This is a generic error, it is raised if an unknown error has been detected.!

XMLRPCError subclass: #XMLRPCClassNotFoundError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XMLRPC-Server-Errors'!

!XMLRPCClassNotFoundError commentStamp: '<historical>' prior: 0!
The request Class wasn't found!

!XMLRPCClassNotFoundError methodsFor: 'as yet unclassified' stamp: ''!
faultCode
	^ 1! !

!XMLRPCError methodsFor: 'accessing' stamp: ''!
faultCode
	"Answer the receiver's 'faultCode'."
	^ faultCode ifNil: [ ^0]! !

!XMLRPCError methodsFor: 'accessing' stamp: ''!
faultCode: anObject
	"Set the receiver's instance variable 'faultCode' to be anObject."

	faultCode := anObject! !

XMLRPCError subclass: #XMLRPCMethodError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XMLRPC-Server-Errors'!

!XMLRPCMethodError methodsFor: 'as yet unclassified' stamp: ''!
faultCode
	^ 2! !

XMLRPCError subclass: #XMLRPCRequestError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XMLRPC-Server-Errors'!

!XMLRPCRequestError commentStamp: '<historical>' prior: 0!
An error raised if a malformed request was found!

!XMLRPCRequestError methodsFor: 'as yet unclassified' stamp: ''!
faultCode
	^ 3! !

XMLRPCError subclass: #XMLRPCReturnError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XMLRPC-Server-Errors'!

!XMLRPCReturnError commentStamp: '<historical>' prior: 0!
An error occured while encoding a methodresponse!

!XMLRPCReturnError methodsFor: 'as yet unclassified' stamp: ''!
faultCode
	^ 4! !

!String methodsFor: '*xmlrpc-server' stamp: 'maf 2/27/2005 23:27'!
asXMLRPCString
	"answer a XMLRPC representation of the receiver"
	| w |
	w _ WriteStream on: String new.
	w nextPutAll: '<value>'; nextPutAll: self asString; nextPutAll: '</value>'.
	^ w contents! !
XMLRPCHttpModule initialize!
XMLRPCServerRequest initialize!
